@@ -20,8 +20,7 @@
 """
 
 from IMAP import IMAPFolder
-import imaplib
-from offlineimap import imaputil, imaplibutil
+from offlineimap import imaputil
 from copy import copy
 
 
