@@ -16,7 +16,9 @@
 #    along with this program; if not, write to the Free Software
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-import re, string, types
+import re
+import string
+import types
 from offlineimap.ui import getglobalui
 quotere = re.compile('^("(?:[^"]|\\\\")*")')
 
@@ -31,7 +33,6 @@ def dequote(string):
     This function does NOT consider parenthised lists to be quoted.
     """
 
-    debug("dequote() called with input:", string)
     if not (string[0] == '"' and string[-1] == '"'):
         return string
     string = string[1:-1]               # Strip off quotes.
@@ -46,7 +47,6 @@ def flagsplit(string):
     return imapsplit(string[1:-1])
 
 def options2hash(list):
-    debug("options2hash called with input:", list)
     retval = {}
     counter = 0
     while (counter < len(list)):
@@ -68,7 +68,6 @@ def imapsplit(imapstring):
 
     ['(\\HasNoChildren)', '"."', '"INBOX.Sent"']"""
 
-    debug("imapsplit() called with input:", imapstring)
     if type(imapstring) != types.StringType:
         debug("imapsplit() got a non-string input; working around.")
         # Sometimes, imaplib will throw us a tuple if the input
@@ -144,7 +143,6 @@ def imapsplit(imapstring):
             elif splitslen == 0:
                 # There was not even an unquoted word.
                 break
-    debug("imapsplit() returning:", retval)
     return retval
             
 flagmap = [('\\Seen', 'S'),
