@@ -16,7 +16,8 @@
 #    along with this program; if not, write to the Free Software
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-import sys, time
+import sys
+import time
 from UIBase import UIBase
 
 class Basic(UIBase):
