@@ -16,20 +16,25 @@
 #    along with this program; if not, write to the Free Software
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-import re, time, sys, traceback, threading, thread
-from StringIO import StringIO
-from Queue import Empty
+import re
+import time
+import sys
+import traceback
+import threading
 import offlineimap
 
-debugtypes = {'imap': 'IMAP protocol debugging',
+debugtypes = {'':'Other offlineimap related sync messages',
+              'imap': 'IMAP protocol debugging',
               'maildir': 'Maildir repository debugging',
               'thread': 'Threading debugging'}
 
 globalui = None
 def setglobalui(newui):
+    """Set the global ui object to be used for logging"""
     global globalui
     globalui = newui
 def getglobalui():
+    """Return the current ui object"""
     global globalui
     return globalui
 
@@ -85,11 +90,14 @@ class UIBase:
                   (threading.currentThread().getName(),
                    s.getthreadaccount(s), account)
         s.threadaccounts[threading.currentThread()] = account
+        s.debug('thread', "Register new thread '%s' (account '%s')" %\
+                    (threading.currentThread().getName(), account))
 
     def unregisterthread(s, thr):
         """Recognizes a thread has exited."""
         if s.threadaccounts.has_key(thr):
             del s.threadaccounts[thr]
+        s.debug('thread', "Unregister thread '%s'" % thr.getName())
 
     def getthreadaccount(s, thr = None):
         if not thr:
@@ -132,7 +140,10 @@ class UIBase:
         raise Exception, "Another OfflineIMAP is running with the same metadatadir; exiting."
 
     def getnicename(s, object):
-        prelimname = str(object.__class__).split('.')[-1]
+        """Return the type of a repository or Folder as string
+
+        (IMAP, Gmail, Maildir, etc...)"""
+        prelimname = object.__class__.__name__.split('.')[-1]
         # Strip off extra stuff.
         return re.sub('(Folder|Repository)', '', prelimname)
 
@@ -257,17 +268,15 @@ class UIBase:
                     ", ".join([str(u) for u in uidlist]),
                     ds))
 
-    def addingflags(s, uidlist, flags, destlist):
+    def addingflags(s, uidlist, flags, dest):
         if s.verbose >= 0:
-            ds = s.folderlist(destlist)
-            s._msg("Adding flags %s to %d messages  on %s" % \
-                   (", ".join(flags), len(uidlist), ds))
+            s._msg("Adding flag %s to %d messages on %s" % \
+                   (", ".join(flags), len(uidlist), dest))
 
-    def deletingflags(s, uidlist, flags, destlist):
+    def deletingflags(s, uidlist, flags, dest):
         if s.verbose >= 0:
-            ds = s.folderlist(destlist)
-            s._msg("Deleting flags %s to %d messages on %s" % \
-                   (", ".join(flags), len(uidlist), ds))
+            s._msg("Deleting flag %s from %d messages on %s" % \
+                   (", ".join(flags), len(uidlist), dest))
 
     ################################################## Threads
 
@@ -299,10 +308,8 @@ class UIBase:
         s.terminate(100)
 
     def getMainExceptionString(s):
-        sbuf = StringIO()
-        traceback.print_exc(file = sbuf)
-        return "Main program terminated with exception:\n" + \
-               sbuf.getvalue() + "\n" + \
+        return "Main program terminated with exception:\n%s\n" %\
+               traceback.format_exc() + \
                s.getThreadDebugLog(threading.currentThread())
 
     def mainException(s):
@@ -331,24 +338,22 @@ class UIBase:
 
     ################################################## Other
 
-    def sleep(s, sleepsecs, siglistener):
+    def sleep(s, sleepsecs, account):
         """This function does not actually output anything, but handles
         the overall sleep, dealing with updates as necessary.  It will,
         however, call sleeping() which DOES output something.
 
-        Returns 0 if timeout expired, 1 if there is a request to cancel
-        the timer, and 2 if there is a request to abort the program."""
-
-        abortsleep = 0
+        :returns: 0/False if timeout expired, 1/2/True if there is a
+                  request to cancel the timer.
+        """
+        abortsleep = False
         while sleepsecs > 0 and not abortsleep:
-            try:
-                abortsleep = siglistener.get_nowait()
-                # retrieved signal while sleeping: 1 means immediately resynch, 2 means immediately die
-            except Empty:
-                # no signal
+            if account.get_abort_event():
+               abortsleep = True
+            else:
                 abortsleep = s.sleeping(10, sleepsecs)
-            sleepsecs -= 10
-        s.sleeping(0, 0)               # Done sleeping.
+                sleepsecs -= 10            
+        s.sleeping(0, 0)  # Done sleeping.
         return abortsleep
 
     def sleeping(s, sleepsecs, remainingsecs):
