@@ -18,14 +18,14 @@
 from offlineimap.ui.UIBase import getglobalui, setglobalui
 from offlineimap.ui import TTY, Noninteractive, Machine
 
-UI_LIST = {'TTY.TTYUI': TTY.TTYUI,
-           'Noninteractive.Basic': Noninteractive.Basic,
-           'Noninteractive.Quiet': Noninteractive.Quiet, 
-           'Machine.MachineUI': Machine.MachineUI}
+UI_LIST = {'ttyui': TTY.TTYUI,
+           'basic': Noninteractive.Basic,
+           'quiet': Noninteractive.Quiet, 
+           'machineui': Machine.MachineUI}
 
 #add Blinkenlights UI if it imports correctly (curses installed)
 try:
     from offlineimap.ui import Curses
-    UI_LIST['Curses.Blinkenlights'] = Curses.Blinkenlights
+    UI_LIST['blinkenlights'] = Curses.Blinkenlights
 except ImportError:
     pass
